@@ -109,24 +109,26 @@ pygi_signal_closure_marshal(GClosure *closure,
         } else if (i < sig_info_highest_arg) {
             GIArgInfo arg_info;
             GITypeInfo type_info;
-            GITransfer transfer;
             GIArgument arg = { 0, };
             PyObject *item = NULL;
             gboolean free_array = FALSE;
 
             g_callable_info_load_arg(signal_info, i - 1, &arg_info);
             g_arg_info_load_type(&arg_info, &type_info);
-            transfer = g_arg_info_get_ownership_transfer(&arg_info);
 
             arg = _pygi_argument_from_g_value(&param_values[i], &type_info);
             
             if (g_type_info_get_tag (&type_info) == GI_TYPE_TAG_ARRAY) {
                 /* Skip the self argument of param_values */
-                arg.v_pointer = _pygi_argument_to_array (&arg, NULL, param_values + 1, signal_info,
-                                                         &type_info, &free_array);
+                arg.v_pointer = _pygi_argument_to_array (&arg,
+                                                         _pygi_argument_array_length_marshal,
+                                                         (void *)(param_values + 1),
+                                                         signal_info,
+                                                         &type_info,
+                                                         &free_array);
             }
             
-            item = _pygi_argument_to_object (&arg, &type_info, transfer);
+            item = _pygi_argument_to_object (&arg, &type_info, GI_TRANSFER_NOTHING);
             
             if (free_array) {
                 g_array_free (arg.v_pointer, FALSE);
